
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */


package chapter10;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.ResultSetMetaData;
import java.sql.DatabaseMetaData;
import javax.sql.DataSource;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import java.util.Iterator;
import java.util.Vector;
import java.io.FileReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.BufferedReader;

public class DynamicQuery {

Connection con      = null;

public static void main( String args[] ) {

String     fileName = null;
String     query    = null;
DynamicQuery dq     = null;

try { 

     //
     // get a local object reference 
     //
     dq = new DynamicQuery();

     // 
     // if we didn't get a command line argument, just use the default file name 
     //
     if ( args.length == 1 ) 
        fileName = args[0];
     else 
        fileName = "query.sql";

    //
    // get the query 
    //
    query = dq.getQuery( fileName );
    if ( query == null ) {
        System.out.println( "No query found. Exiting ... " );
        System.exit( -1 );
     }

    //
    // process the query 
    //
    System.out.println("****************************");
    System.out.println("Processing file: " + fileName );
    System.out.println("Processing Query: " + query );
    System.out.println("****************************");

     //
     // pass the connection into the processQuery method. 
     // it is responsible for executing the query 
     //
     ResultSet rs = dq.processQuery( dq.con,  query );

     //
     // convert ResultSet data to Java types 
     //
     Vector data     = dq.convertResultSet( rs );
     Vector colNames = dq.getColumnNames( rs );
   
     //
     // output ResultSet 
     //
     // need to get back to the first row since
     // convertResult method has moved the pointer 
     //
     rs.beforeFirst( );
     if ( rs != null ) {
         dq.outputResultSet( rs );
     }
     else {
         System.out.println( "The query did not return results.");
     }

     //
     // output converted data 
     //
     if ( data != null ) {
         dq.outputData( data, colNames );
     }
     else {
         System.out.println( "The query did not return results.");
     }

				
}
catch (Exception e) {
     System.out.println("Exception in main: " + e.getMessage() );
     e.printStackTrace();
}
finally {

    try {   
        // close the connection 
	if ( dq.con != null )
            dq.con.close(); 
    }
    catch (SQLException e) {
          e.printStackTrace();
    }
}

}
// ------------------------------------------------------------------
public DynamicQuery() {
try { 
     //
     // JNDI startup parameters are stored in the "jndi.properties" file in the 
     // classpath.
     //
     InitialContext ctx = new InitialContext( );

     //
     // get the DataSource from the JNDI name server 
     //
     DataSource ds = (DataSource) ctx.lookup("movies");

     //
     // get the connection from the DataSource
     //
     con = ds.getConnection( );
}
catch (NamingException e ) {
      System.out.println( "NamingException in DynamicQuery: " + e.getMessage() );
}
catch (SQLException e) {
      System.out.println( "SQLException in DynamicQuery: " + e.getMessage() );
}
catch (Exception e) {
      System.out.println( "Exception in DynamicQuery: " + e.getMessage() );
}

}
// -------------------------------------------------------------------
public ResultSet processQuery( String query ) {

     return processQuery( con, query );

}

// -------------------------------------------------------------------
public ResultSet  processQuery( Connection con, String query ) {

ResultSet rs = null;
try {

Statement stmt = con.createStatement();

rs = stmt.executeQuery( query );

}

catch (SQLException e ) {
     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in processQuery: " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();
}
finally {
  return rs;
}


}


// -------------------------------------------------------------------
void outputResultSet( ResultSet rs ) {

try { 

     ResultSetMetaData resMetaData = rs.getMetaData();
     int colCount = resMetaData.getColumnCount();

     while ( rs.next() ) {

            //
            // 
	    //
	    System.out.println( "**********************************" );
	    for (int n = 1; n <= colCount; n++ ) {
	         System.out.println( resMetaData.getColumnLabel( n )  + ": " + "\t\t" + 
				     rs.getString( n ) + 
				     //" - class: " + resMetaData.getColumnClassName( n ) +
				     " - type: " +  resMetaData.getColumnTypeName( n )  +
				     " - precision: " +  resMetaData.getPrecision( n ) +
				     " - scale: " +  resMetaData.getScale( n ) 
				     );                     

	         //System.out.println( resMetaData.getColumnName( n )  + ": " + "\t\t" + 
		//		     rs.getString( n ) );                     

            }
	    System.out.println( "**********************************" );

     }
}
catch (SQLException e ) {
     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in outputResultSet: " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();
}

}

// ------------------------------------------------------------------
public String getQuery( String fn ) {
String buffer  = null; // data buffer 
String retVal  = null; // query string to return 

try { 
     //
     // create a file reader for the file containing the query
     //
     BufferedReader reader = new BufferedReader ( new FileReader( fn ));

     // 
     // start reading the file 
     //
     buffer = reader.readLine();
     while ( buffer != null ) {

          if ( retVal == null ) {   // this is the first pass 
              retVal = buffer;
           }
          else {
              retVal += buffer;     // append 
          }
      //
      // continue reading the file 
      //
      buffer = reader.readLine();
     }

    //
    // close the file reader 
    //
    reader.close();
}
catch (FileNotFoundException e) {
     System.out.println("FileNotFoundException in getQuery: " + e.getMessage() );
}
catch (IOException e) { 
     System.out.println("IOException in getQuery: " + e.getMessage() );
}
finally { 
     return retVal;
}

}
// ---------------------------------------------------------------------

public int getRows( ResultSet rs ) {
int rows = 0;
try { 

    while ( rs.next() ) {
            rows++;
    }

}
catch (SQLException e) {
      System.out.println("SQLException in getRows: " + e.getMessage() );
}
finally {
    return rows;
}

}
// ---------------------------------------------------------------------
public Vector convertResultSet( ResultSet rs ) {
// convert the ResultSet data into an array of Objects 
// and store in a Vector 
Vector data = new Vector(); 

try {

 while ( rs.next() ) {

      Vector row  = new Vector(); 
      for ( int n = 1; n <= rs.getMetaData().getColumnCount(); n++ ) {

           // return an Object reference for Java wrapper types 
           // row.add( JDBCTypeMapper.getColumnData( n, rs ) );         
           row.add( JDBCTypeMapper.getColumnDataString( n, rs ) );         
      }

      data.add( row );
 }
}
catch (SQLException e) {
      System.out.println("SQLException in getRows: " + e.getMessage() );
}
finally {
  return data;
}

}

// --------------------------------------------------------------------

public Vector getColumnNames( ResultSet rs ) {
Vector colNames        = new Vector();


try {

  ResultSetMetaData rsmd = rs.getMetaData();

  for ( int n = 1; n <= rsmd.getColumnCount(); n++ ) {
        colNames.add( rsmd.getColumnName( n ) );
  }

}
catch (SQLException e) {
   System.out.println("SQLException in getColumnNames: " + e.getMessage() );
}
finally {
  return colNames;
}

}

// ---------------------------------------------------------------------

void outputData( Vector data, Vector colNames ) {
Iterator i = data.iterator();

     while ( i.hasNext() ) {

            //
            // use the iterator to move through the rows 
	    //
	    Vector   row = (Vector) i.next();
            int colCount = row.size();
	    System.out.println( "**********************************" );

	    //
	    // print the column data for the row 
	    //
	    for (int n = 0; n < colCount; n++ ) {
	         System.out.println( colNames.get( n )  + 
		                   " - (" + row.get( n).getClass().getName() + "): " +
				   "\t\t" + row.get( n ) );   
            }
	    System.out.println( "**********************************" );

     }

}
// ---------------------------------------------------------------------

} // end class 





